package dust

import (
	"context"
	"fmt"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"strings"
	"time"
)

// 获取分钟 06:10 06:11
func getMinute(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}

	array := strings.Split(tmp[1], ":")
	if len(array) < 3 {
		return t
	}
	return fmt.Sprintf("%s:%s", array[0], array[1])
}

// 获取小时 06:00 07:00
func getHour(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}
	array := strings.Split(tmp[1], ":")
	if len(array) < 3 {
		return t
	}
	return fmt.Sprintf("%s:%s", array[0], "00")
}

// 获取天 1号 2号
func getDay(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}
	array := strings.Split(tmp[0], "-")
	if len(array) < 3 {
		return t
	}
	day := array[2]
	if day[0] == '0' {
		day = string(day[1])
	}
	return fmt.Sprintf("%s号", day)
}

// 根据类型获取时间
func getTimeByType(t string, reqType int32) string {
	switch reqType {
	case 0:
		return getMinute(t)
	case 1:
		return getHour(t)
	case 2:
		return getDay(t)
	}
	return getMinute(t)
}

func getTimeArray(reqTime int64, reqType int32) []string {
	layout := "2006-01-02 15:04:05"
	start := time.Unix(reqTime, 0)
	ret := []string{}

	switch reqType {
	case 0:
		startHour := start.Hour()
		for {
			if start.Hour() != startHour {
				break
			}
			minute := getMinute(start.Format(layout))
			ret = append(ret, minute)
			start = start.Add(1 * time.Minute)
		}
	case 1:
		startDay := start.Day()
		for {
			if start.Day() != startDay {
				break
			}
			hour := getHour(start.Format(layout))
			ret = append(ret, hour)
			start = start.Add(60 * time.Minute)
		}
	case 2:
		startMonth := start.Month()
		for {
			if start.Month() != startMonth {
				break
			}
			day := getDay(start.Format(layout))
			ret = append(ret, day)
			start = start.Add(24 * 60 * time.Minute)
		}
	}
	return ret
}

func getStartAndEnd(reqTime int64, reqType int32) (string, string) {
	layout := "2006-01-02 15:04:05"
	start := time.Unix(reqTime, 0)
	end := start
	switch reqType {
	case 0:
		end = start.Add(60 * time.Minute)
	case 1:
		end = start.Add(24 * 60 * time.Minute)
	case 2:
		for {
			end = end.Add(24 * 60 * time.Minute)
			if end.Month() != start.Month() {
				break
			}
		}
	default:
		end = start.Add(60 * time.Minute)
	}
	return start.Format(layout), end.Format(layout)
}


func DustMonitorHistoryFromInfluxdb(ctx context.Context, req *v1.DustMonitorHistoryRequest) (reply *v1.DustMonitorHistoryReply, err error) {
	reply = &v1.DustMonitorHistoryReply{}
	start, end := getStartAndEnd(req.Time, req.Type)
	dataType := consts.DustDataTypeMinute
	switch req.Type {
	case 0:
		dataType = consts.DustDataTypeMinute
	case 1:
		dataType = consts.DustDataTypeHour
	case 2:
		dataType = consts.DustDataTypeTotalDay
	}
	array := []model.DustMonitorInfo{}
	sql := fmt.Sprintf("select * from \"%s\" where time >= '%s' and time < '%s' and Type=%d tz('Asia/Shanghai')", req.Sn, start, end, dataType)
	_, err = model.QueryInfluxdb(sql, "DustMonitorInfo", &array)
	if err != nil {
		return nil, errors.DataBaseError
	}
	

	m := map[string][]v1.DustMonitorData{}

	// 获取时段坐标
	tarray := getTimeArray(req.Time, req.Type)
	for _, v := range tarray {
		m[v] = []v1.DustMonitorData{}
	}

	for i, v := range array {
		item := v1.DustMonitorData{}
		item.Date = getTimeByType(v.Time, req.Type)
		item.B03 = array[i].Voice
		item.H01 = array[i].Humidity
		item.P01 = array[i].Atmospheric
		item.Pm10 = array[i].Pm10Value
		item.Pm25 = array[i].Pm25Value
		item.R01 = array[i].Rain
		item.T01 = array[i].Temperature
		item.Tsp = array[i].Tsp
		item.W01 = array[i].WindDirection
		item.W02 = array[i].WindSpeed
		m[item.Date] = append(m[item.Date], item)
	}

	// 一个时段如果有多条数据算平均值
	reply.List = make([]*v1.DustMonitorData, len(tarray))
	for i, k := range tarray {
		subarray := m[k]
		if len(subarray) == 0 {
			reply.List[i] = &v1.DustMonitorData{Date: k}

			continue
		}
		newItem := v1.DustMonitorData{}
		length := len(subarray)
		for _, v := range subarray {
			newItem.Date = v.Date
			newItem.B03 += v.B03
			newItem.H01 += v.H01
			newItem.P01 += v.P01
			newItem.Pm10 += v.Pm10
			newItem.Pm25 += v.Pm25
			newItem.R01 += v.R01
			newItem.T01 += v.T01
			newItem.Tsp += v.Tsp
			newItem.W01 += v.W01
			newItem.W02 += v.W02
		}
		newItem.B03 = newItem.B03 / float64(length)
		newItem.H01 = newItem.H01 / float64(length)
		newItem.P01 = newItem.P01 / float64(length)
		newItem.Pm10 = newItem.Pm10 / float64(length)
		newItem.Pm25 = newItem.Pm25 / float64(length)
		newItem.R01 = newItem.R01 / float64(length)
		newItem.T01 = newItem.T01 / float64(length)
		newItem.Tsp = newItem.Tsp / float64(length)
		newItem.W01 = newItem.W01 / float64(length)
		newItem.W02 = newItem.W02 / float64(length)
		reply.List[i] = &newItem
	}

	return reply, nil
}

func DustMonitorHistory(ctx context.Context, req *v1.DustMonitorHistoryRequest) (reply *v1.DustMonitorHistoryReply, err error) {

	if req.Time == 0 || req.Sn == "" {
		return nil, errors.ParamsError
	}
	return DustMonitorHistoryFromInfluxdb(ctx, req)
/*
	reply = &v1.DustMonitorHistoryReply{}
	start, end := getStartAndEnd(req.Time, req.Type)
	p := model.DustMonitorInfo{}
	where := map[string]interface{}{
		"SN":req.Sn,
		"MonitoringTime >=": start,
		"MonitoringTime <":end,
	}
	dataType := consts.DustDataTypeMinute
	switch req.Type {
	case 0:
		dataType = consts.DustDataTypeMinute
	case 1:
		dataType = consts.DustDataTypeHour
	case 2:
		dataType = consts.DustDataTypeDay
	}

	where["Type"] = dataType

	array, err := p.All(database.DB(), where)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	m := map[string][]v1.DustMonitorData{}

	// 获取时段坐标
	tarray := getTimeArray(req.Time, req.Type)
	for _, v := range tarray {
		m[v] = []v1.DustMonitorData{}
	}

	for i, v := range array {
		item := v1.DustMonitorData{}
		item.Date = getTimeByType(v.MonitoringTime.Format("2006-01-02 15:04:05"), req.Type)
		item.B03 = array[i].Voice
		item.H01 = array[i].Humidity
		item.P01 = array[i].Atmospheric
		item.Pm10 = array[i].Pm10Value
		item.Pm25 = array[i].Pm25Value
		item.R01 = array[i].Rain
		item.T01 = array[i].Temperature
		item.Tsp = array[i].Tsp
		item.W01 = array[i].WindDirection
		item.W02 = array[i].WindSpeed
		m[item.Date] = append(m[item.Date], item)

	}

	// 一个时段如果有多条数据算平均值
	reply.List = make([]*v1.DustMonitorData, len(tarray))
	for i, k := range tarray {
		subarray := m[k]
		if len(subarray) == 0 {
			reply.List[i] = &v1.DustMonitorData{Date: k}

			continue
		}
		newItem := v1.DustMonitorData{}
		length := len(subarray)
		for _, v := range subarray {
			newItem.Date = v.Date
			newItem.B03 += v.B03
			newItem.H01 += v.H01
			newItem.P01 += v.P01
			newItem.Pm10 += v.Pm10
			newItem.Pm25 += v.Pm25
			newItem.R01 += v.R01
			newItem.T01 += v.T01
			newItem.Tsp += v.Tsp
			newItem.W01 += v.W01
			newItem.W02 += v.W02
		}
		newItem.B03 = newItem.B03 / float64(length)
		newItem.H01 = newItem.H01 / float64(length)
		newItem.P01 = newItem.P01 / float64(length)
		newItem.Pm10 = newItem.Pm10 / float64(length)
		newItem.Pm25 = newItem.Pm25 / float64(length)
		newItem.R01 = newItem.R01 / float64(length)
		newItem.T01 = newItem.T01 / float64(length)
		newItem.Tsp = newItem.Tsp / float64(length)
		newItem.W01 = newItem.W01 / float64(length)
		newItem.W02 = newItem.W02 / float64(length)
		reply.List[i] = &newItem
	}

	return reply, nil

 */
}
