package lift

import (
	"context"
	"go.uber.org/zap"
	"smart-site-management/model"
	"fmt"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"strconv"
	"time"
)

const (
	LiftAlarmHigh   = "L01"
	LiftAlarmWeight = "L02"
	LiftAlarmPeople = "L03"
	LiftAlarmSpeed  = "L04"
	LiftAlarmWind   = "L05"
	LiftAlarmTilt   = "L06"

	LiftWarningHigh    = "L07"
	LiftWarningWeight  = "L08"
	LiftWarningPeople  = "L09"
	LiftWarningSpeed   = "L10"
	LiftWarningWind    = "L11"
	LiftWarningTilt    = "L12"
	LiftAlarmFrontDoor = "L13"
	LiftAlarmBackDoor  = "L14"
)

type LiftRealtimeData struct {
	DateTime            int64   `json:"date_time"`
	Weight              int64   `json:"weight"`
	WeightPercent       int64   `json:"weight_percent"`
	People              int64   `json:"people"`
	High                float64 `json:"high"`
	HighPercent         int64   `json:"high_percent"`
	Speed               float64 `json:"speed"`
	Direction           int     `json:"direction"`
	Tilt1               float64 `json:"tilt1"`
	Tilt1Percent        int64   `json:"tilt1_percent"`
	Tilt2               float64 `json:"tilt2"`
	Tilt2Percent        int64   `json:"tilt2_percent"`
	DriverNo            int64   `json:"driver_no"`
	FrontDoorState      int     `json:"front_door_state"`
	BackDoorState       int     `json:"back_door_state"`
	WeightAlarmState    int     `json:"weight_alarm_state"`
	HighAlarmState      int     `json:"high_alarm_state"`
	SpeedAlarmState     int     `json:"speed_alarm_state"`
	PeopleAlarmState    int     `json:"people_alarm_state"`
	TiltAlarmState      int     `json:"tilt_alarm_state"`
	FrontDoorAlarmState int     `json:"front_door_alarm_state"`
	BackDoorAlarmState  int     `json:"back_door_alarm_state"`
	WindSpeedAlarmState int     `json:"wind_speed_alarm_state"`
	WindSpeed           float64 `json:"wind_speed"`
	Lng        float64 `json:"lng"`
	Lat        float64 `json:"lat"`
	Sn         string  `json:"sn"`
	ProjectId  int64   `json:"project_id"`
	LiftNo     int64   `json:"lift_no"`
	LiftDirect int     `json:"lift_direct"` //0 左吊笼 1 右吊笼
	IsAlarm int `json:"is_alarm"`
}



var LiftAlarmCodeReasonMap = map[string]string{
	"L01":"高度报警",
	"L02":"重量报警",
	"L03":"人数报警",
	"L04":"速度报警",
	"L05":"风速报警",
	"L06":"倾角报警",

	"L07":"高度预警",
	"L08":"重量预警",
	"L09":"人数预警",
	"L10":"速度预警",
	"L11":"风速预警",
	"L12":"倾角预警",
	"L13":"前门异常",
	"L14":"后门异常",
}


func judgeAlarmCode(value int, codes ...string) string {
	if value == 0 {
		return ""
	}
	if value == 1 && len(codes) > 0 {
		return codes[0]
	}
	if value == 2 && len(codes) > 1 {
		return codes[1]
	}
	return ""
}

func getAlarmCodes(record *LiftRealtimeData) []string {
	ret := []string{}
	if code := judgeAlarmCode(record.HighAlarmState, LiftWarningHigh, LiftAlarmHigh); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.PeopleAlarmState, LiftWarningPeople, LiftAlarmPeople); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.WeightAlarmState, LiftWarningWeight, LiftAlarmWeight); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.WindSpeedAlarmState, LiftWarningWind, LiftAlarmWind); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.TiltAlarmState, LiftWarningTilt, LiftAlarmTilt); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.SpeedAlarmState, LiftWarningSpeed, LiftAlarmSpeed); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.FrontDoorAlarmState,  LiftAlarmFrontDoor); code != "" {
		ret = append(ret, code)
	}
	if code := judgeAlarmCode(record.BackDoorAlarmState, LiftAlarmBackDoor); code != "" {
		ret = append(ret, code)
	}
	return ret
}

func getLiftHistorySql(req *v1.LiftMonitorHistoryRequest) (string, string, string) {
	//layout := "2006-01-02 15:04:05"
	sql := fmt.Sprintf("select * from \"%s\" ", req.Sn)
	where := ""
	whereArray := []string{}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("date_time >= %d ", req.Start))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("date_time < %d ", req.End))
	}
	if req.IsAlarm {
		whereArray = append(whereArray, "is_alarm > 0")
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	offset := int64(req.Page - 1) * int64(req.PageSize)
	countSql := fmt.Sprintf("select count(high) as count from \"%s\" %s tz('Asia/Shanghai')", req.Sn, where)
	sql = fmt.Sprintf("%s %s order by time limit %d offset %d tz('Asia/Shanghai')", sql, where, req.PageSize, offset)
	fmt.Printf("sql:%s\n", sql)
	return sql, countSql, "lift_monitor_realtime_data"
}

func getLiftHistoryCount(dbname string, sql string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err := model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}
	//fmt.Printf("aaaaa:%s,%s,%v\n", sql, dbname, array)
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Count, nil
}

func floatParse(data float64) float64 {
	str := strconv.FormatFloat(data, 'f', 2, 64)
	ret, _ := strconv.ParseFloat(str, 64)
	return ret
}

func LiftMonitorHistory(ctx context.Context, req *v1.LiftMonitorHistoryRequest) (reply *v1.LiftMonitorHistoryReply, err error) {
	if req.Sn == "" {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}


	reply = &v1.LiftMonitorHistoryReply{
		Page:req.Page,
		PageSize:req.PageSize,
	}

	sql, countSql, dbname := getLiftHistorySql(req)
	if err != nil {
		logger.Error("LiftMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	total, err := getLiftHistoryCount(dbname, countSql)
	if err != nil {
		logger.Error("LiftMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	reply.Total = total
	if reply.Total == 0 {
		return reply, nil
	}

	array := []LiftRealtimeData{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("LiftMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}


	for i, v := range array {
		item := v1.LiftMonitorData{}
		item.Date = time.Unix(v.DateTime, 0).Format("2006-01-02 15:04:05")
		item.Speed = v.Speed
		item.Weight = v.Weight
		item.High = v.High
		item.AngleX = v.Tilt1
		item.AngleY = v.Tilt2
		if v.BackDoorState > 0 {
			item.BackDoor = true
		}
		if v.FrontDoorState > 0 {
			item.FrontDoor = true
		}
		item.SpeedDirection = int32(v.Direction)
		item.LeftRight = int32(v.LiftDirect)
		codes := getAlarmCodes(&array[i])
		for _, code := range codes {
			if reason, ok := LiftAlarmCodeReasonMap[code]; ok {
				item.AlarmReasons = append(item.AlarmReasons, reason)
			}
		}

		reply.List = append(reply.List, &item)
	}

	return reply, nil
}
