// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package model

import (
	"encoding/json"
	"fmt"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
)

type TAlarm struct {
	Id          int64  `gorm:"column:ID;primary_key"`
	ProjectId   int64  `gorm:"column:ProjectId"`
	Sn          string `gorm:"column:SN"`
	AlarmCode   string `gorm:"column:AlarmCode"`
	AlarmReason string `gorm:"column:AlarmReason"`
	IsHandle    bool   `gorm:"column:IsHandle"`
	Time        time.Time `gorm:"column:Time"`
	CreatedAt   string `gorm:"column:CreatedAt"`
	UpdatedAt   string `gorm:"column:UpdatedAt"`
	DeviceCode uint32 `gorm:"column:DeviceCode"`
	First bool `gorm:"column:First"`
}

func (TAlarm) TableName() string {
	return "Alarm"
}

func (p *TAlarm) Insert(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}
	return nil
}

func (p *TAlarm) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}
	return nil
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *TAlarm) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	filed["UpdatedAt"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := json.Marshal(filed)
		logger.Error("mysql",
			zap.String("sql", "update t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}
	return nil
}

func (p *TAlarm) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return errors.DataBaseError
	}
	return nil
}

func (p *TAlarm) QueryAll(db *gorm.DB, filter map[string]interface{}) (list []TAlarmContact, err error) {
	err = db.Where(filter).Find(&list).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return list, errors.DataBaseError
	}
	return list, nil
}

func (p *TAlarm) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count from t_alarm"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return ret, errors.DataBaseError
		}
		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select count from t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return ret, errors.DataBaseError
	}
	return ret, err
}

type AlarmListRequest struct {
	Sn string
	Page int32
	PageSize int32
	IsHandle int32
	Start int64
	End int64
	Timestamp int64
	ProjectId int64
	DeviceCode int32
}

type AlarmListItem struct {
	TAlarm
	DeviceCode int32
	Name string
}

func AlarmListMoreSql(req *AlarmListRequest)(string, string) {
	sql := fmt.Sprintf("select t1.*, t2.DeviceCode as device_code, t2.DeviceName as name from Alarm as t1 left join DeviceAll as t2 on t1.SN=t2.SN and t1.ProjectId=t2.ProjectId and t1.DeviceCode=t2.DeviceCode")
	countSql := fmt.Sprintf("select count(1) as count  from Alarm as t1 left join DeviceAll as t2 on t1.SN=t2.SN and t1.ProjectId=t2.ProjectId and t1.DeviceCode=t2.DeviceCode")
	whereArray := []string{}
	where := ""
	whereArray = append(whereArray, fmt.Sprintf("t1.First = 1"))
	if req.Sn != "" {
		whereArray = append(whereArray, fmt.Sprintf("t1.Sn='%s'", req.Sn))
	}
	if req.DeviceCode > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.DeviceCode=%d", req.DeviceCode))
	}
	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", req.ProjectId))
	}
	if req.Timestamp > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.CreatedAt<'%s'", time.Unix(req.Timestamp, 0).Format("2006-01-02 15:04:05")))
	}
	if req.IsHandle == 1 {
		whereArray = append(whereArray, fmt.Sprintf("t1.IsHandle=1"))
	} else if req.IsHandle == 2 {
		whereArray = append(whereArray, fmt.Sprintf("t1.IsHandle=0"))
	}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.Time >= '%s'", time.Unix(req.Start, 0).Format("2006-01-02 15:04:05")))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.Time < '%s'", time.Unix(req.End, 0).Format("2006-01-02 15:04:05")))
	}

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := (req.Page - 1) *req.PageSize
	limit := req.PageSize

	sql = fmt.Sprintf("%s %s order by t1.Time desc limit %d offset %d", sql, where, limit, offset)
	countSql = fmt.Sprintf("%s %s", countSql, where)

	return sql, countSql
}

func (p *TAlarm) ListMore(db *gorm.DB, req *AlarmListRequest) (list []AlarmListItem, total int64, err error) {

	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []AlarmListItem{}

	sql, countSql := AlarmListMoreSql(req)

	err = db.Raw(countSql).Scan(&array).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}

func (p *TAlarm) List(db *gorm.DB, where map[string]interface{}, page int32, pagesize int32) (list []TAlarm, err error) {
	offset := (page - 1) * pagesize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_alarm"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Limit(10).Offset(page).Find(&list).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select * from t_alarm"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return list, errors.DataBaseError
	}

	return list, err

}
